;*****************************************************
;* $RCSfile$
;* $Revision$
;* $Author$ at $Date$
;* Description:	Common macros for all AVR processors.
;		Note:	AVRGenericDef.inc should be included
;			before this file is included.
;******************************************************
; This code is public domain however its author claim no responsibility
; for it or its use.
; See ReadMe for more details.
; http://www.avrml.sourcesafe.net

;**************************************************************************
;	Macro bris	(Branch if I/O register Set)
; 	Expects: @0-I/O address, @1 - bit to test, @2 branch destination
;	Uses:	
;	Words:	2
;	Cycles: 3/4
;
;	If the bit in the I/O register is set it branches otherwise continues.
;**************************************************************************
.macro bris
	sbic	@0, @1
	rjmp	@2
.endmacro
;**************************************************************************
;	/bris
;**************************************************************************

;**************************************************************************
;	Macro bric	(Branch if I/O register Cleared)
; 	Expects: @0-I/O address, @1 - bit to test, @2 branch destination
;	Uses:	
;	Words:	2
;	Cycles: 3/4
;
;	If the bit in the I/O register is clear it branches otherwise continues.
;**************************************************************************
.macro bric
	sbis	@0, @1
	rjmp	@2
.endmacro
;**************************************************************************
;	/bric
;**************************************************************************

;**************************************************************************
;	Macro brwh	(Branch if word higher)
; 	Expects: @0-word register pair to compare, @1- Word register pair
;		to compare. @2 - Place to branch
;	Uses:	
;	Words:	5
;	Cycles: 3/4/6/7
;
;	Compares the two words. If @0 is lower or equal to @1 it doesn't branch.
;**************************************************************************
.macro brwh
	cp	@1H, @0H
	brlo	@2
	brne	PC+3	
	cp	@1L, @0L	; PC + 1
	brlo	@2		; PC + 2
				; PC + 3
.endmacro
;**************************************************************************
;	/brwh
;**************************************************************************

;**************************************************************************
;	Macro stw	(Store Word)
; 	Expects: @0-X, Y, Z (must have + and -), @1- Register pair to store
;	Uses:	
;	Words:	2
;	Cycles: 4
;
;	Stores the word pair to the pointer. It can have post increment
;	and decrement. Stores high byte first.
;**************************************************************************
.macro	stw
	st	@0, @1L
	st	@0, @1H
.endmacro
;**************************************************************************
;	/stw
;**************************************************************************

;**************************************************************************
;	Macro ldw	(Load Word)
; 	Expects: @0- Register pair to store, @1-X, Y, Z (must have + and -)
;	Uses:	
;	Words:	2
;	Cycles: 4
;	
;	Loads the word pointed to by the pointer.
;**************************************************************************
.macro	ldw
	ld	@0L, @1
	ld	@0H, @1
.endmacro
;**************************************************************************
;	/ldw
;**************************************************************************
					
;**************************************************************************
;	Macro ldiw	(Load Immediate Word)
; 	Expects: @0-word register pair to load into, @1- constant to load
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Loads the constant word into the register. Can be used to load pointers.
;**************************************************************************
.macro	ldiw
	ldi	@0L, low(@1)
	ldi	@0H, high(@1)
.endmacro
;**************************************************************************
;	/ldiw
;**************************************************************************

;**************************************************************************
;	Macro mvw	(Move Word)
; 	Expects: @0-word register pair to to move to, @1- from
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Moves the word to the other registers
;	Note: On some processors their is movw which you should use if they
;	have it.
;**************************************************************************
; Now we have to put the count in from the right side
.macro	mvw
	mov	@0L, @1L
	mov	@0H, @1H
.endmacro
;**************************************************************************
;	/mvw
;**************************************************************************

;**************************************************************************
;	Macro brwz	(Branch if word zero)
; 	Expects: @0-word register pair to test, @1- address to branch to 
;	Uses:	
;	Words:	4
;	Cycles: 3/4/5
;
;	Tests the word and branches if it's zero
;**************************************************************************
.macro	brwz
	tst	@0L
	brne	PC+3	; breq Zero
	tst	@0H	; PC + 1
	breq	@1	; PC + 2
;Zero:			; PC + 3
.endmacro
;**************************************************************************
;	/brwz
;**************************************************************************

;**************************************************************************
;	Macro brwnz	(Branch if word not zero)
; 	Expects: @0-word register pair to test, @1- address to branch to 
;	Uses:	
;	Words:	4
;	Cycles: 3/4/5
;
;	Tests the word and branches if it's not zero
;**************************************************************************
.macro	brwnz
	tst	@0L
	brne	@1
	tst	@0H	; PC + 1
	brne	@1	; PC + 2
;Zero:			; PC + 3
.endmacro
;**************************************************************************
;	/brwnz
;**************************************************************************

;**************************************************************************
;	Macro brze	(Branch if zero)
; 	Expects: @0 - branch to if zero  
;	Uses:	
;	Words:	1
;	Cycles: 1/2
;
;	Same as breq just more obvious what you're doing.
;**************************************************************************
.macro	brze
	breq	@0
.endmacro
;**************************************************************************
;	/brze
;**************************************************************************

;**************************************************************************
;	Macro brnz	(Branch if not zero)
; 	Expects: @0 - branch to if not zero  
;	Uses:	
;	Words:	1
;	Cycles: 1/2
;
;	Same as brnz just more obvious what you're doing.
;**************************************************************************
.macro	brnz
	brne	@0
.endmacro
;**************************************************************************
;	/brnz
;**************************************************************************

;**************************************************************************
;	Macro brrs	(Branch if bit in register set)
; 	Expects: @0 register, @1 bit, @2 - branch to if set
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Branches if the bit in the register is set. It should be a bit 
;	number not a mask.
;**************************************************************************
.macro	brrs
	sbrc	@0, @1
	rjmp	@2
.endmacro
;**************************************************************************
;	/brrs
;**************************************************************************

;**************************************************************************
;	Macro brrc	(Branch if bit in register cleared)
; 	Expects: @0 register, @1 bit, @2 - branch to if cleared
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Branches if the bit in the register is cleared. It should be a bit 
;	number not a mask.
;**************************************************************************
.macro	brrc
	sbrs	@0, @1
	rjmp	@2
.endmacro
;**************************************************************************
;	/brrc
;**************************************************************************

;**************************************************************************
;	Macro lbrze	(Long branch if zero)
; 	Expects: @0 - branch to if zero
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brze just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrze
	brne	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrze
;**************************************************************************

;**************************************************************************
;	Macro lbrnz	(Long branch if not zero)
; 	Expects: @0 - branch to if not zero
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brnz just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrnz
	breq	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrnz
;**************************************************************************

;**************************************************************************
;	Macro lbrts	(Long branch if T set)
; 	Expects: @0 - branch to if T set
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brts just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrts
	brtc	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrts
;**************************************************************************

;**************************************************************************
;	Macro lbrtc	(Long branch if T cleared)
; 	Expects: @0 - branch to if T cleared
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brtc just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrtc
	brts	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrtc
;**************************************************************************

;**************************************************************************
;	Macro lbrcs	(Long branch if Carry set)
; 	Expects: @0 - branch to if C set
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brcs just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrcs
	brcc	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrcs
;**************************************************************************

;**************************************************************************
;	Macro lbrcc	(Long branch if Carry cleared)
; 	Expects: @0 - branch to if C cleared
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brcc just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrcc
	brcs	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrcc
;**************************************************************************

;**************************************************************************
;	Macro lbreq	(Long branch if equal)
; 	Expects: @0 - branch to if equal
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as breq just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbreq
	brne	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbreq
;**************************************************************************

;**************************************************************************
;	Macro lbrne	(Long branch if not equal)
; 	Expects: @0 - branch to if not equal
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brne just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrne
	breq	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrne
;**************************************************************************

;**************************************************************************
;	Macro lbrlo	(Long branch if lower)
; 	Expects: @0 - branch to if lower
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brlo just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrlo
	brsh	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrlo
;**************************************************************************

;**************************************************************************
;	Macro lbrsh	(Long branch if same or higher)
; 	Expects: @0 - branch to if same or higher
;	Uses:	
;	Words:	2
;	Cycles: 2/3
;
;	Same as brsh just can go longer (as far as an rjmp)
;**************************************************************************
.macro	lbrsh
	brlo	PC+2
	rjmp	@0
.endmacro
;**************************************************************************
;	/lbrsh
;**************************************************************************

;**************************************************************************
;	Macro stsw	(Store to SRAM word)
; 	Expects: @0- address, @1- register pair, 
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Stores the word from the SRAM address. Words are stored MSB last.
;**************************************************************************
.macro	stsw
	sts	@0 + 1, @1H
	sts	@0, @1L
.endmacro
;**************************************************************************
;	/stsw
;**************************************************************************

;**************************************************************************
;	Macro clrw	(Clear Word)
; 	Expects: @0 - register pair, 
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Clears the register pair.
;**************************************************************************
.macro	clrw
	clr	@0L
	clr	@0H
.endmacro
;**************************************************************************
;	/clrw
;**************************************************************************

;**************************************************************************
;	Macro ldsw	(Load from SRAM word)
; 	Expects: @0- register pair, @1- address
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Loads the word from the SRAM address. Words are stored MSB last.
;**************************************************************************
.macro	ldsw
	lds	@0H, @1 + 1
	lds	@0L, @1
.endmacro
;**************************************************************************
;	/ldsw
;**************************************************************************

;**************************************************************************
;	Macro outw	(Output to I/O address word)
; 	Expects: @0- address, @1- register pair
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Outputs the data to the address (high byte first).
;**************************************************************************
.macro	outw
	out	@0H, @1H
	out	@0L, @1L
.endmacro
;**************************************************************************
;	/outw
;**************************************************************************

;**************************************************************************
;	Macro inw	(Input to I/O address word)
; 	Expects: @0- register pair, @1- address
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Gets the data to the address (high byte first).
;**************************************************************************
.macro	inw
	in	@0H, @1H
	in	@0L, @1L
.endmacro
;**************************************************************************
;	/inw
;**************************************************************************

;**************************************************************************
;	Macro pushw	(Push word)
; 	Expects: @0- register pair
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Pushes a word onto the stack (push the low byte first).
;**************************************************************************
.macro	pushw
	push	@0L
	push	@0H
.endmacro
;**************************************************************************
;	/pushw
;**************************************************************************

;**************************************************************************
;	Macro popw	(Pop word)
; 	Expects: @0- register pair
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Pushes a word onto the stack (pop the high byte first)
;**************************************************************************
.macro	popw
	pop	@0H
	pop	@0L
.endmacro
;**************************************************************************
;	/popw
;**************************************************************************

;**************************************************************************
;	Macro addi	(Add Immediate)
; 	Expects: @0- register, @1 Add operand
;	Uses:	
;	Words:	1
;	Cycles: 1
;
;	Adds the immediate operand. It is done by subtracting the negative
;	of the number.
;**************************************************************************
.macro	addi
	subi	@0, -(@1)
.endmacro
;**************************************************************************
;	/addi
;**************************************************************************

;**************************************************************************
;	Macro aiw	(Add Immediate Word)
; 	Expects: @0- register pair, @2 Add operand
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Adds the immediate operand. It is done by subtracting the negative
;	of the number. You should use adiw if the operand is less than
;	64 and you are using one of the 4 high register pairs.
;**************************************************************************
.macro	aiw
	subi	@0L, (low(-@1))   ; am 13.7.2010 gendert A.Moeller
	sbci	@0H, (high(-@1))  ; am 13.7.2010 gendert A.Moeller
.endmacro
;**************************************************************************
;	/aiw
;**************************************************************************

;**************************************************************************
;	Macro siw	(Subtract Immediate Word)
; 	Expects: @0- register pair, @1 Subtract operand
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Subtracts the immediate operand.
;	You should use adiw if the operand is less than
;	64 and you are using one of the 4 high register pairs.
;**************************************************************************
.macro	siw
	subi	@0L, low(@1)
	sbci	@0H, high(@1)
.endmacro
;**************************************************************************
;	/siw
;**************************************************************************

;**************************************************************************
;	Macro addw	(Add Word)
; 	Expects: @0- register pair for result, @1 register pair
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Adds the two words together.
;**************************************************************************
.macro	addw
	add	@0L, @1L
	adc	@0H, @1H
.endmacro
;**************************************************************************
;	/addw
;**************************************************************************

;**************************************************************************
;	Macro subw	(Subtract Word)
; 	Expects: @0- register pair for result, @1 register pair
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Subtracts the two words.
;**************************************************************************
.macro 	subw
	sub	@0L, @1L
	sbc	@0H, @1H
.endmacro
;**************************************************************************
;	/subw
;**************************************************************************

;**************************************************************************
;	Macro SaveSREG
; 	Expects: -
;	Uses:	RLStatusRegSave
;	Words:	1
;	Cycles: 1
;
;	Toggles the status register for an interupt.
;**************************************************************************
.macro	SaveSREG
	in	RLStatusRegSave, SREG
.endmacro
;**************************************************************************
;	/SaveSREG
;**************************************************************************

;**************************************************************************
;	Macro RestoreSREGreti
; 	Expects: SaveSREG to have been previously called.
;	Uses:	RLStatusRegSave
;	Words:	1
;	Cycles: 5	
;
;	Used at the end of an ISR to restore the status register and return.
;**************************************************************************
.macro	RestoreSREGreti
	out	SREG, RLStatusRegSave
	reti
.endmacro
;**************************************************************************
;	/RestoreSREGreti
;**************************************************************************

;**************************************************************************
;	Macro ToggleC
; 	Expects:
;	Uses:
;	Words:	4
;	Cycles: 3/4	
;
;	Toggles C flag.
;**************************************************************************
.macro	ToggleC
	brcs	PC+4
	sec
	rjmp	PC+2
	clc
.endmacro
;**************************************************************************
;	/ToggleC
;**************************************************************************

;**************************************************************************
;	Macro ToggleT
; 	Expects:
;	Uses:
;	Words:	4
;	Cycles: 3/4	
;
;	Toggles T flag.
;**************************************************************************
.macro	ToggleT
	brts	PC+3
	set
	rjmp	PC+2
	clt
.endmacro
;**************************************************************************
;	/ToggleT
;**************************************************************************
